using UnityEngine;
using System;
using System.Collections;
using System.Collections.Generic;
using SimpleJSON;
using System.Linq;
using System.IO;

// CUAEditor example
//
// Base editor for a CUA (Custom Unity Asset)
namespace CUAEditorPlugin
{
    public class CUAEditor : MVRScript
    {
		// My JSONStorable for the editor
		public JSONStorableColor SphereColor;
		public JSONStorableBool SphereVisibility;
		public JSONStorableColor CubeColor;
		public JSONStorableBool CubeVisibility;
		
		// My misc variables to find and edit the prefab made in Unity
		private string objectNameSearchString = "vam_script_tutorial";
		public Transform ObjectRoot;
		private Transform sphereTR;
		private Transform cubeTR;
		private Renderer sphereRenderer;
		private Renderer cubeRenderer;
	
		// Security and initialization
		private bool editorInitialized = false;
		private int initLoops = 0;
		private int maxInitLoops = 3600;

		// The usual init function for VAM
		public override void Init()
        {			 
            try
            {
				// We only authorize the script to be used by a CUA
				if (containingAtom.type != "CustomUnityAsset")
                {
                    logError("CUAEditor - Please add this script on a Custom Unity Asset ( script disabled ).");
                    return;
                }
				
				// Default color for both shapes
				HSVColor hsvc = HSVColorPicker.RGBToHSV(1f, 1f, 1f);
				
				// Color selection for the sphere
				SphereColor = new JSONStorableColor("Sphere color", hsvc, SphereColorCallback);
				RegisterColor(SphereColor);
				CreateColorPicker(SphereColor);
				
				// Visibility state for the sphere
				SphereVisibility = new JSONStorableBool("Sphere is visible", true, SphereVisibilityCallback);
				SphereVisibility.storeType = JSONStorableParam.StoreType.Full;
				CreateToggle(SphereVisibility, false);
				RegisterBool(SphereVisibility);
				
				// Color selection for the cube
				CubeColor = new JSONStorableColor("Cube color", hsvc, CubeColorCallback);
				RegisterColor(CubeColor);
				CreateColorPicker(CubeColor);
				
				// Visibility state for the cube
				CubeVisibility = new JSONStorableBool("Cube is visible", true, CubeVisibilityCallback);
				CubeVisibility.storeType = JSONStorableParam.StoreType.Full;
				CreateToggle(CubeVisibility, false);
				RegisterBool(CubeVisibility);

				// A tiny bit of help to give some info to your fellow creator and/or player
				JSONStorableString helpText = new JSONStorableString("Help",
					"<color=#000><size=35><b>Help</b></size></color>\n\n" + 
					"<color=#333>" +
					"<b>CUAEditor</b> is a tiny plugin to show you how you can create an editor for a CUA ( Custom Unity Asset ).\n\n" +
					"Change the parameters on the left and you will see the CUA updating in realtime.\n\n" +
					"<b>IMPORTANT :</b> If you switch objects in the Asset tab of the CUA, don't forget to reload the script to enable customization.\n\n" +
					"</color>"
				);
				UIDynamic helpTextfield = CreateTextField(helpText, true);
				helpTextfield.height = 1100.0f;
			}
            catch(Exception e)
            {
                SuperController.LogError("CUAEditor - Exception caught: " + e);
            }
		}
		
		// In this case we don't need that, but maybe you do ?
		void Start()
        {

        }
		
		// Used for the initialization of the editor.
		// When it is initialized, there's nothing done every frame
		void Update()
        {
			// If the editor is not initialized, we're gonna try to do it
			// I'm putting 3600 tries ( at 60fps it is 1min )
			if( initLoops < maxInitLoops && editorInitialized == false ) {
				initLoops++;
				InitEditor();
			}
			
			// Triggering an error if we have reached our limit of tries
			if( initLoops == maxInitLoops ) {
				logError("CUAEditor : This CUA could not be initialized properly after 3600 tries. Please select a compatible object and reload the script.");
				initLoops++; // To avoid spamming the log with the error
			}
		}
		
		// The function initializing everything we need to update the prefab with the editor
		// You can imagine whatever you want here, the only limitations are the things available in VAM's build
		// In this example case, we need the main transform, a transform for the sphere and the cube
		//
		// Initially this function is made to return a bool state to see if it is properly initialized.
		// But since the CUA loader is asynchronous when you reload a scene and I don't have a way to identify when unity's prefab is loaded
		// I have to "brute force" the initialization several times until the script finds the transform I need
		// It implies that your prefab is done properly and has the good name ( this is why the maxInitLoops exists )
		private bool InitEditor() {
			// When the editor is initialized, we don't need to do that anymore
			if( editorInitialized == true ) return true;
			
			// We need to find our main transform, getObjectRoot is gonna do that for us
			// the search is based on our variable objectNameSearchString
			// you need to create a prefab containing this name in Unity
			// This method works for unparented and parented objects
			// ( Thanks for the reParentObject tip meshed ! )
			ObjectRoot = getObjectRoot( containingAtom.reParentObject );
			if( ObjectRoot == null ) {
				return false;
			}
			
			// Finding my sphere and cube transforms
			sphereTR = ObjectRoot.Find("sphere");
			cubeTR = ObjectRoot.Find("cube");
					
			// I found my transforms, now the rest
			if( sphereTR && cubeTR ) {
				// Finding my sphere and cube renderers
				sphereRenderer = sphereTR.GetComponent<Renderer>();
				cubeRenderer = cubeTR.GetComponent<Renderer>();
				
				// I found my renderers components, let's finish this
				if( sphereRenderer && cubeRenderer ) {
					// Everything is good, I can consider my editor initialized
					editorInitialized = true;
					
					// I now need initialize my different objects based on the informations in my JSONStorables
					// You could go crazy and make unified functions to control the whole object instead
					SphereColorCallback( SphereColor );
					SphereVisibilityCallback( SphereVisibility.val );
					CubeColorCallback( CubeColor );
					CubeVisibilityCallback( CubeVisibility.val );
								
					logDebug("CUAEditor : editor initialized properly");
					return true;
				} else {
					logError("CUAEditor : your CUA has a transform with the good name but does not have a renderer.");
					initLoops = maxInitLoops; // We don't have what we need, we consider the initialization failed
					return false;
				}
			} else {
				logError("CUAEditor : your CUA is missing mandatory components ( transform name cube and sphere ).");
				initLoops = maxInitLoops; // We don't have what we need, we consider the initialization failed
				return false;
			}
			
			return false;
		}
		
		// Recursive function searching for a transform containing the string set in objectNameSearchString
		// It is used the first time for the initialization of the script
		private Transform getObjectRoot( Transform parent ) {
			foreach( Transform child in parent )
			{
				if( child.name.Contains(objectNameSearchString) ) {
					return child;
				} else {
					Transform childSearch = getObjectRoot(child);
					if (childSearch != null) return childSearch;
				}
			}
			
			return null;
		}
		
		// Callback triggered when you change the color for the sphere (or you can call it manually)
		protected void SphereColorCallback(JSONStorableColor color) {
			if( sphereRenderer ) {
				Color newColor = new Color(color.colorPicker.currentColor.r, color.colorPicker.currentColor.g, color.colorPicker.currentColor.b, 1.0f);
				sphereRenderer.material.SetColor("_Color",newColor);
			}
		}
		
		// Callback triggered when you change the visibility for the sphere (or you can call it manually)
		protected void SphereVisibilityCallback(bool visibility) {
			if( sphereTR ) {
				sphereTR.gameObject.SetActive(visibility);
			}
		}
		
		// Callback triggered when you change the color for the cube (or you can call it manually)
		protected void CubeColorCallback(JSONStorableColor color) {
			if( cubeRenderer ) {
				Color newColor = new Color(color.colorPicker.currentColor.r, color.colorPicker.currentColor.g, color.colorPicker.currentColor.b, 1.0f);
				cubeRenderer.material.SetColor("_Color",newColor);
			}
		}
		
		// Callback triggered when you change the visibility for the sphere (or you can call it manually)
		protected void CubeVisibilityCallback(bool visibility) {
			if( cubeTR ) {
				cubeTR.gameObject.SetActive(visibility);
			}
		}
		
		// **************************
		// Local Tools... just because I'm lazy
		// **************************
		private void logDebug( string debugText ) {
			SuperController.LogMessage( debugText );
		}
		
		private void logError( string debugText ) {
			SuperController.LogError( debugText );
		}
	}
}